package jdk.internal.org.objectweb.asm.tree;

import java.util.NoSuchElementException;
import java.util.ListIterator;
import jdk.internal.org.objectweb.asm.MethodVisitor;

public class InsnList
{
    private int size;
    private AbstractInsnNode first;
    private AbstractInsnNode last;
    AbstractInsnNode[] cache;
    
    public InsnList() {
    }
    
    public int size() {
        return this.size;
    }
    
    public AbstractInsnNode getFirst() {
        return this.first;
    }
    
    public AbstractInsnNode getLast() {
        return this.last;
    }
    
    public AbstractInsnNode get(int n) {
        if (n < 0 || n >= this.size) {
            throw new IndexOutOfBoundsException();
        }
        if (this.cache == null) {
            this.cache = this.toArray();
        }
        return this.cache[n];
    }
    
    public boolean contains(AbstractInsnNode abstractInsnNode) {
        AbstractInsnNode abstractInsnNode2;
        for (abstractInsnNode2 = this.first; abstractInsnNode2 != null && abstractInsnNode2 != abstractInsnNode; abstractInsnNode2 = abstractInsnNode2.next) {}
        return abstractInsnNode2 != null;
    }
    
    public int indexOf(AbstractInsnNode abstractInsnNode) {
        if (this.cache == null) {
            this.cache = this.toArray();
        }
        return abstractInsnNode.index;
    }
    
    public void accept(MethodVisitor methodVisitor) {
        for (AbstractInsnNode abstractInsnNode = this.first; abstractInsnNode != null; abstractInsnNode = abstractInsnNode.next) {
            abstractInsnNode.accept(methodVisitor);
        }
    }
    
    public ListIterator<AbstractInsnNode> iterator() {
        return this.iterator(0);
    }
    
    public ListIterator<AbstractInsnNode> iterator(int n) {
        return new InsnList.InsnListIterator(n);
    }
    
    public AbstractInsnNode[] toArray() {
        int n = 0;
        AbstractInsnNode abstractInsnNode = this.first;
        AbstractInsnNode[] array = new AbstractInsnNode[this.size];
        while (abstractInsnNode != null) {
            array[n] = abstractInsnNode;
            abstractInsnNode.index = n++;
            abstractInsnNode = abstractInsnNode.next;
        }
        return array;
    }
    
    public void set(AbstractInsnNode abstractInsnNode, AbstractInsnNode abstractInsnNode2) {
        AbstractInsnNode next = abstractInsnNode.next;
        abstractInsnNode2.next = next;
        if (next != null) {
            next.prev = abstractInsnNode2;
        }
        else {
            this.last = abstractInsnNode2;
        }
        AbstractInsnNode prev = abstractInsnNode.prev;
        abstractInsnNode2.prev = prev;
        if (prev != null) {
            prev.next = abstractInsnNode2;
        }
        else {
            this.first = abstractInsnNode2;
        }
        if (this.cache != null) {
            int index = abstractInsnNode.index;
            this.cache[index] = abstractInsnNode2;
            abstractInsnNode2.index = index;
        }
        else {
            abstractInsnNode2.index = 0;
        }
        abstractInsnNode.index = -1;
        abstractInsnNode.prev = null;
        abstractInsnNode.next = null;
    }
    
    public void add(AbstractInsnNode abstractInsnNode) {
        ++this.size;
        if (this.last == null) {
            this.first = abstractInsnNode;
            this.last = abstractInsnNode;
        }
        else {
            this.last.next = abstractInsnNode;
            abstractInsnNode.prev = this.last;
        }
        this.last = abstractInsnNode;
        this.cache = null;
        abstractInsnNode.index = 0;
    }
    
    public void add(InsnList list) {
        if (list.size == 0) {
            return;
        }
        this.size += list.size;
        if (this.last == null) {
            this.first = list.first;
            this.last = list.last;
        }
        else {
            AbstractInsnNode first = list.first;
            this.last.next = first;
            first.prev = this.last;
            this.last = list.last;
        }
        this.cache = null;
        list.removeAll(false);
        return;
    }
    
    public void insert(AbstractInsnNode abstractInsnNode) {
        ++this.size;
        if (this.first == null) {
            this.first = abstractInsnNode;
            this.last = abstractInsnNode;
        }
        else {
            this.first.prev = abstractInsnNode;
            abstractInsnNode.next = this.first;
        }
        this.first = abstractInsnNode;
        this.cache = null;
        abstractInsnNode.index = 0;
    }
    
    public void insert(InsnList list) {
        if (list.size == 0) {
            return;
        }
        this.size += list.size;
        if (this.first == null) {
            this.first = list.first;
            this.last = list.last;
        }
        else {
            AbstractInsnNode last = list.last;
            this.first.prev = last;
            last.next = this.first;
            this.first = list.first;
        }
        this.cache = null;
        list.removeAll(false);
        return;
    }
    
    public void insert(AbstractInsnNode prev, AbstractInsnNode next) {
        ++this.size;
        AbstractInsnNode next2 = prev.next;
        if (next2 == null) {
            this.last = next;
        }
        else {
            next2.prev = next;
        }
        prev.next = next;
        next.next = next2;
        next.prev = prev;
        this.cache = null;
        next.index = 0;
    }
    
    public void insert(AbstractInsnNode prev, InsnList list) {
        if (list.size == 0) {
            return;
        }
        this.size += list.size;
        AbstractInsnNode first = list.first;
        AbstractInsnNode last = list.last;
        AbstractInsnNode next = prev.next;
        if (next == null) {
            this.last = last;
        }
        else {
            next.prev = last;
        }
        prev.next = first;
        last.next = next;
        first.prev = prev;
        this.cache = null;
        list.removeAll(false);
        return;
    }
    
    public void insertBefore(AbstractInsnNode next, AbstractInsnNode prev) {
        ++this.size;
        AbstractInsnNode prev2 = next.prev;
        if (prev2 == null) {
            this.first = prev;
        }
        else {
            prev2.next = prev;
        }
        next.prev = prev;
        prev.next = next;
        prev.prev = prev2;
        this.cache = null;
        prev.index = 0;
    }
    
    public void insertBefore(AbstractInsnNode next, InsnList list) {
        if (list.size == 0) {
            return;
        }
        this.size += list.size;
        AbstractInsnNode first = list.first;
        AbstractInsnNode last = list.last;
        AbstractInsnNode prev = next.prev;
        if (prev == null) {
            this.first = first;
        }
        else {
            prev.next = first;
        }
        next.prev = last;
        last.next = next;
        first.prev = prev;
        this.cache = null;
        list.removeAll(false);
        return;
    }
    
    public void remove(AbstractInsnNode abstractInsnNode) {
        --this.size;
        AbstractInsnNode next = abstractInsnNode.next;
        AbstractInsnNode prev = abstractInsnNode.prev;
        if (next == null) {
            if (prev == null) {
                this.first = null;
                this.last = null;
            }
            else {
                prev.next = null;
                this.last = prev;
            }
        }
        else if (prev == null) {
            this.first = next;
            next.prev = null;
        }
        else {
            prev.next = next;
            next.prev = prev;
        }
        this.cache = null;
        abstractInsnNode.index = -1;
        abstractInsnNode.prev = null;
        abstractInsnNode.next = null;
    }
    
    void removeAll(boolean b) {
        if (b) {
            AbstractInsnNode next;
            for (AbstractInsnNode first = this.first; first != null; first = next) {
                next = first.next;
                first.index = -1;
                first.prev = null;
                first.next = null;
            }
        }
        this.size = 0;
        this.first = null;
        this.last = null;
        this.cache = null;
    }
    
    public void clear() {
        this.removeAll(false);
    }
    
    public void resetLabels() {
        for (AbstractInsnNode abstractInsnNode = this.first; abstractInsnNode != null; abstractInsnNode = abstractInsnNode.next) {
            if ((abstractInsnNode instanceof LabelNode)) {
                ((LabelNode)abstractInsnNode).resetLabel();
            }
        }
    }
    
    private final class InsnListIterator implements ListIterator
    {
        AbstractInsnNode next;
        AbstractInsnNode prev;
        AbstractInsnNode remove;
        
        InsnListIterator(int n) {
            if (n == InsnList.this.size()) {
                this.next = null;
                this.prev = InsnList.this.getLast();
            }
            else {
                this.next = InsnList.this.get(n);
                this.prev = this.next.prev;
            }
        }
        
        @Override
        public boolean hasNext() {
            return this.next != null;
        }
        
        @Override
        public Object next() {
            if (this.next == null) {
                throw new NoSuchElementException();
            }
            AbstractInsnNode next = this.next;
            this.prev = next;
            this.next = next.next;
            return this.remove = next;
        }
        
        @Override
        public void remove() {
            if (this.remove != null) {
                if (this.remove == this.next) {
                    this.next = this.next.next;
                }
                else {
                    this.prev = this.prev.prev;
                }
                InsnList.this.remove(this.remove);
                this.remove = null;
                return;
            }
            else {
                throw new IllegalStateException();
            }
        }
        
        @Override
        public boolean hasPrevious() {
            return this.prev != null;
        }
        
        @Override
        public Object previous() {
            AbstractInsnNode prev = this.prev;
            this.next = prev;
            this.prev = prev.prev;
            return this.remove = prev;
        }
        
        @Override
        public int nextIndex() {
            if (this.next == null) {
                return InsnList.this.size();
            }
            if (InsnList.this.cache == null) {
                InsnList.this.cache = InsnList.this.toArray();
            }
            return this.next.index;
        }
        
        @Override
        public int previousIndex() {
            if (this.prev == null) {
                return -1;
            }
            if (InsnList.this.cache == null) {
                InsnList.this.cache = InsnList.this.toArray();
            }
            return this.prev.index;
        }
        
        @Override
        public void add(Object o) {
            InsnList.this.insertBefore(this.next, (AbstractInsnNode)o);
            this.prev = (AbstractInsnNode)o;
            this.remove = null;
        }
        
        @Override
        public void set(Object o) {
            InsnList.this.set(this.next.prev, (AbstractInsnNode)o);
            this.prev = (AbstractInsnNode)o;
        }
    }
}
